@@ -55,10 +55,6 @@ typedef struct {
 	double scale;
 } NautilusCanvasPosition;
 
-typedef enum {
-	NAUTILUS_CANVAS_LABEL_POSITION_UNDER,
-} NautilusCanvasLabelPosition;
-
 #define	NAUTILUS_CANVAS_CONTAINER_TYPESELECT_FLUSH_DELAY 1000000
 
 typedef struct NautilusCanvasContainerDetails NautilusCanvasContainerDetails;
@@ -137,8 +133,7 @@ typedef struct {
 	NautilusIconInfo *(* get_icon_images)     (NautilusCanvasContainer *container,
 						     NautilusCanvasIconData *data,
 						     int canvas_size,
-						     gboolean for_drag_accept,
-						     gboolean *has_window_open);
+						     gboolean for_drag_accept);
 	void         (* get_icon_text)            (NautilusCanvasContainer *container,
 						     NautilusCanvasIconData *data,
 						     char **editable_text,
@@ -152,8 +147,6 @@ typedef struct {
 	int          (* compare_icons_by_name)    (NautilusCanvasContainer *container,
 						     NautilusCanvasIconData *canvas_a,
 						     NautilusCanvasIconData *canvas_b);
-	void         (* freeze_updates)           (NautilusCanvasContainer *container);
-	void         (* unfreeze_updates)         (NautilusCanvasContainer *container);
 	void         (* prioritize_thumbnailing)  (NautilusCanvasContainer *container,
 						   NautilusCanvasIconData *data);
 
@@ -202,11 +195,6 @@ typedef struct {
 	void         (* icon_position_changed)    (NautilusCanvasContainer *container,
 						     NautilusCanvasIconData *data,
 						     const NautilusCanvasPosition *position);
-	void         (* icon_rename_started)      (NautilusCanvasContainer *container,
-						     GtkWidget *renaming_widget);
-	void         (* icon_rename_ended)        (NautilusCanvasContainer *container,
-						     NautilusCanvasIconData *data,
-						     const char *text);
 	void	     (* icon_stretch_started)     (NautilusCanvasContainer *container,
 						     NautilusCanvasIconData *data);
 	void	     (* icon_stretch_ended)       (NautilusCanvasContainer *container,
@@ -259,8 +247,6 @@ void              nautilus_canvas_container_set_auto_layout               (Nauti
 gboolean          nautilus_canvas_container_is_keep_aligned               (NautilusCanvasContainer  *container);
 void              nautilus_canvas_container_set_keep_aligned              (NautilusCanvasContainer  *container,
 									   gboolean                keep_aligned);
-void              nautilus_canvas_container_set_label_position            (NautilusCanvasContainer  *container,
-									   NautilusCanvasLabelPosition pos);
 void              nautilus_canvas_container_sort                          (NautilusCanvasContainer  *container);
 void              nautilus_canvas_container_freeze_icon_positions         (NautilusCanvasContainer  *container);
 
@@ -288,8 +274,6 @@ gboolean          nautilus_canvas_container_has_stretch_handles           (Nauti
 gboolean          nautilus_canvas_container_is_stretched                  (NautilusCanvasContainer  *container);
 void              nautilus_canvas_container_show_stretch_handles          (NautilusCanvasContainer  *container);
 void              nautilus_canvas_container_unstretch                     (NautilusCanvasContainer  *container);
-void              nautilus_canvas_container_start_renaming_selected_item  (NautilusCanvasContainer  *container,
-									   gboolean                select_all);
 
 /* options */
 NautilusCanvasZoomLevel nautilus_canvas_container_get_zoom_level                (NautilusCanvasContainer  *view);
