@@ -17,8 +17,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   see <http://www.gnu.org/licenses/>.
 
    Authors: Pavel Cisler <pavel@eazel.com>,
    	    Ettore Perazzoli <ettore@gnu.org>
@@ -117,19 +116,6 @@ nautilus_drag_destroy_selection_list (GList *list)
 	g_list_free (list);
 }
 
-char **
-nautilus_drag_uri_array_from_selection_list (const GList *selection_list)
-{
-	GList *uri_list;
-	char **uris;
-
-	uri_list = nautilus_drag_uri_list_from_selection_list (selection_list);
-	uris = nautilus_drag_uri_array_from_list (uri_list);
-	g_list_free_full (uri_list, g_free);
-
-	return uris;
-}
-
 GList *
 nautilus_drag_uri_list_from_selection_list (const GList *selection_list)
 {
@@ -148,26 +134,6 @@ nautilus_drag_uri_list_from_selection_list (const GList *selection_list)
 	return g_list_reverse (uri_list);
 }
 
-char **
-nautilus_drag_uri_array_from_list (const GList *uri_list)
-{
-	const GList *l;
-	char **uris;
-	int i;
-
-	if (uri_list == NULL) {
-		return NULL;
-	}
-
-	uris = g_new0 (char *, g_list_length ((GList *) uri_list));
-	for (i = 0, l = uri_list; l != NULL; l = l->next) {
-		uris[i++] = g_strdup ((char *) l->data);
-	}
-	uris[i] = NULL;
-
-	return uris;
-}
-
 GList *
 nautilus_drag_uri_list_from_array (const char **uris)
 {
@@ -322,17 +288,6 @@ nautilus_drag_items_local (const char *target_uri_string,
 						  ((NautilusDragSelectionItem *)selection_list->data)->uri);
 }
 
-gboolean
-nautilus_drag_items_in_trash (const GList *selection_list)
-{
-	/* check if the first item on the list is in trash.
-	 * FIXME:
-	 * we should really test each item but that would be slow for large selections
-	 * and currently dropped items can only be from the same container
-	 */
-	return eel_uri_is_trash (((NautilusDragSelectionItem *)selection_list->data)->uri);
-}
-
 gboolean
 nautilus_drag_items_on_desktop (const GList *selection_list)
 {
@@ -664,41 +619,6 @@ nautilus_drag_drag_data_get_from_cache (GList *cache,
 	return TRUE;
 }
 
-gboolean
-nautilus_drag_drag_data_get (GtkWidget *widget,
-			GdkDragContext *context,
-			GtkSelectionData *selection_data,
-			guint info,
-			guint32 time,
-			gpointer container_context,
-			NautilusDragEachSelectedItemIterator each_selected_item_iterator)
-{
-	GString *result;
-		
-	switch (info) {
-	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
-		result = g_string_new (NULL);
-		(* each_selected_item_iterator) (add_one_gnome_icon, container_context, result);
-		break;
-		
-	case NAUTILUS_ICON_DND_URI_LIST:
-	case NAUTILUS_ICON_DND_TEXT:
-		result = g_string_new (NULL);
-		(* each_selected_item_iterator) (add_one_uri, container_context, result);
-		break;
-
-	default:
-		return FALSE;
-	}
-	
-	gtk_selection_data_set (selection_data,
-				gtk_selection_data_get_target (selection_data),
-				8, (guchar *) result->str, result->len);
-	g_string_free (result, TRUE);
-
-	return TRUE;
-}
-
 typedef struct
 {
 	GMainLoop *loop;
@@ -786,11 +706,6 @@ nautilus_drag_drop_action_ask (GtkWidget *widget,
 				      (actions & GDK_ACTION_LINK) != 0,
 				      &damd);
 
-	append_drop_action_menu_item (menu, _("Set as _Background"),
-				      NAUTILUS_DND_ACTION_SET_AS_BACKGROUND,
-				      (actions & NAUTILUS_DND_ACTION_SET_AS_BACKGROUND) != 0,
-				      &damd);
-
 	eel_gtk_menu_append_separator (GTK_MENU (menu));
 	
 	menu_item = gtk_menu_item_new_with_mnemonic (_("Cancel"));
