@@ -30,14 +30,6 @@
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-string.h>
 
-#define NAUTILUS_FILE_LARGE_TOP_LEFT_TEXT_MAXIMUM_CHARACTERS_PER_LINE 80
-#define NAUTILUS_FILE_LARGE_TOP_LEFT_TEXT_MAXIMUM_LINES               24
-#define NAUTILUS_FILE_LARGE_TOP_LEFT_TEXT_MAXIMUM_BYTES               10000
-
-#define NAUTILUS_FILE_TOP_LEFT_TEXT_MAXIMUM_CHARACTERS_PER_LINE 10
-#define NAUTILUS_FILE_TOP_LEFT_TEXT_MAXIMUM_LINES               5
-#define NAUTILUS_FILE_TOP_LEFT_TEXT_MAXIMUM_BYTES               1024
-
 #define NAUTILUS_FILE_DEFAULT_ATTRIBUTES				\
 	"standard::*,access::*,mountable::*,time::*,unix::*,owner::*,selinux::*,thumbnail::*,id::filesystem,trash::orig-path,trash::deletion-date,metadata::*"
 
@@ -61,6 +53,7 @@ struct NautilusFileDetails
 
 	eel_ref_str display_name;
 	char *display_name_collation_key;
+	char *directory_name_collation_key;
 	eel_ref_str edit_name;
 
 	goffset size; /* -1 is unknown */
@@ -99,9 +92,11 @@ struct NautilusFileDetails
 	char *thumbnail_path;
 	GdkPixbuf *thumbnail;
 	time_t thumbnail_mtime;
-	
+
+	GdkPixbuf *scaled_thumbnail;
+	double thumbnail_scale;
+
 	GList *mime_list; /* If this is a directory, the list of MIME types in it. */
-	char *top_left_text;
 
 	/* Info you might get from a link (.desktop, .directory or nautilus link) */
 	GIcon *custom_icon;
@@ -168,10 +163,6 @@ struct NautilusFileDetails
 
 	eel_boolean_bit mount_is_up_to_date           : 1;
 	
-	eel_boolean_bit got_top_left_text             : 1;
-	eel_boolean_bit got_large_top_left_text       : 1;
-	eel_boolean_bit top_left_text_is_up_to_date   : 1;
-
 	eel_boolean_bit got_link_info                 : 1;
 	eel_boolean_bit link_info_is_up_to_date       : 1;
 	eel_boolean_bit got_custom_display_name       : 1;
@@ -184,9 +175,6 @@ struct NautilusFileDetails
 	
 	eel_boolean_bit is_thumbnailing               : 1;
 
-	/* TRUE if the file is open in a spatial window */
-	eel_boolean_bit has_open_window               : 1;
-
 	eel_boolean_bit is_launcher                   : 1;
 	eel_boolean_bit is_trusted_link               : 1;
 	eel_boolean_bit is_foreign_link               : 1;
@@ -240,11 +228,7 @@ NautilusFile *nautilus_file_new_from_info                  (NautilusDirectory
 							    GFileInfo              *info);
 void          nautilus_file_emit_changed                   (NautilusFile           *file);
 void          nautilus_file_mark_gone                      (NautilusFile           *file);
-char *        nautilus_extract_top_left_text               (const char             *text,
-							    gboolean                large,
-							    int                     length);
-void          nautilus_file_set_directory                  (NautilusFile           *file,
-							    NautilusDirectory      *directory);
+
 gboolean      nautilus_file_get_date                       (NautilusFile           *file,
 							    NautilusDateType        date_type,
 							    time_t                 *date);
@@ -270,14 +254,11 @@ gboolean      nautilus_file_set_display_name               (NautilusFile
 							    const char             *display_name,
 							    const char             *edit_name,
 							    gboolean                custom);
+void          nautilus_file_set_directory                  (NautilusFile           *file,
+							    NautilusDirectory      *directory);
 void          nautilus_file_set_mount                      (NautilusFile           *file,
 							    GMount                 *mount);
 
-/* Return true if the top lefts of files in this directory should be
- * fetched, according to the preference settings.
- */
-gboolean      nautilus_file_should_get_top_left_text       (NautilusFile           *file);
-
 /* Mark specified attributes for this file out of date without canceling current
  * I/O or kicking off new I/O.
  */
