@@ -43,7 +43,21 @@ struct NautilusSearchDirectoryDetails {
 	NautilusSearchEngine *engine;
 
 	gboolean search_running;
-	gboolean search_loaded;
+        /* When the search directory is stopped or cancelled, we migth wait
+         * until all data and signals from previous search are stopped and removed
+         * from the search engine. While this situation happens we don't want to connect
+         * clients to our signals, and we will wait until the search data and signals
+         * are valid and ready.
+         * The worst thing that can happens if we don't do this is that new clients
+         * migth get the information of old searchs if they are waiting_for_file_list.
+         * But that shouldn't be a big deal since old clients have the old information.
+         * But anyway it's currently unused for this case since the only client is
+         * nautilus-view and is not waiting_for_file_list :) .
+         *
+         * The other use case is for letting clients know if information of the directory
+         * is outdated or not valid. This might happens for automatic
+         * scheduled timeouts. */
+	gboolean search_ready_and_valid;
 
 	GList *files;
 	GHashTable *files_hash;
@@ -159,7 +173,7 @@ start_search (NautilusSearchDirectory *search)
 
 	/* We need to start the search engine */
 	search->details->search_running = TRUE;
-	search->details->search_loaded = FALSE;
+	search->details->search_ready_and_valid = FALSE;
 
 	set_hidden_files (search);
 	nautilus_search_provider_set_query (NAUTILUS_SEARCH_PROVIDER (search->details->engine),
@@ -438,11 +452,11 @@ search_call_when_ready (NautilusDirectory *directory,
 	search_callback->wait_for_attributes = file_attributes;
 	search_callback->wait_for_file_list = wait_for_file_list;
 
-	if (wait_for_file_list && !search->details->search_loaded) {
+	if (wait_for_file_list && !search->details->search_ready_and_valid) {
 		/* Add it to the pending callback list, which will be
-		 * processed when the directory has finished loading
-		 */
-		search->details->pending_callback_list = 
+		 * processed when the directory has valid data from the new
+                 * search and all data and signals from previous searchs is removed. */
+		search->details->pending_callback_list =
 			g_list_prepend (search->details->pending_callback_list, search_callback);
 
 		/* We might need to start the search engine */
@@ -478,8 +492,8 @@ search_cancel_callback (NautilusDirectory *directory,
 		search->details->callback_list = g_list_remove (search->details->callback_list, search_callback);
 		
 		search_callback_destroy (search_callback);
-		
-		return;
+
+	        goto done;
 	} 
 
 	/* Check for a pending callback */
@@ -489,11 +503,12 @@ search_cancel_callback (NautilusDirectory *directory,
 		search->details->pending_callback_list = g_list_remove (search->details->pending_callback_list, search_callback);
 
 		search_callback_destroy (search_callback);
-
-		if (!search->details->pending_callback_list) {
-			stop_search (search);
-		}
 	}
+
+done:
+        if (!search->details->callback_list && !search->details->pending_callback_list) {
+	        stop_search (search);
+        }
 }
 
 static void
@@ -506,15 +521,8 @@ search_callback_add_pending_file_callbacks (SearchCallback *callback)
 }
 
 static void
-search_directory_ensure_loaded (NautilusSearchDirectory *search)
+search_directory_add_pending_files_callbacks (NautilusSearchDirectory *search)
 {
-	if (search->details->search_loaded) {
-		return;
-	}
-
-	search->details->search_loaded = TRUE;
-	nautilus_directory_emit_done_loading (NAUTILUS_DIRECTORY (search));
-
 	/* Add all file callbacks */
 	g_list_foreach (search->details->pending_callback_list,
 			(GFunc)search_callback_add_pending_file_callbacks, NULL);
@@ -525,6 +533,13 @@ search_directory_ensure_loaded (NautilusSearchDirectory *search)
 	search->details->pending_callback_list = NULL;
 }
 
+static void
+on_search_directory_search_ready_and_valid (NautilusSearchDirectory *search)
+{
+	search_directory_add_pending_files_callbacks (search);
+        search->details->search_ready_and_valid = TRUE;
+}
+
 static void
 search_engine_hits_added (NautilusSearchEngine *engine, GList *hits, 
 			  NautilusSearchDirectory *search)
@@ -573,7 +588,7 @@ search_engine_hits_added (NautilusSearchEngine *engine, GList *hits,
 	nautilus_file_emit_changed (file);
 	nautilus_file_unref (file);
 
-	search_directory_ensure_loaded (search);
+        on_search_directory_search_ready_and_valid (search);
 }
 
 static void
@@ -589,9 +604,26 @@ search_engine_error (NautilusSearchEngine *engine, const char *error_message, Na
 }
 
 static void
-search_engine_finished (NautilusSearchEngine *engine, NautilusSearchDirectory *search)
+search_engine_finished (NautilusSearchEngine         *engine,
+                        NautilusSearchProviderStatus  status,
+                        NautilusSearchDirectory      *search)
 {
-	search_directory_ensure_loaded (search);
+        /* If the search engine is going to restart means it finished an old search
+         * that was stopped or cancelled.
+         * Don't emit the done loading signal in this case, since this means the search
+         * directory tried to start a new search before all the search providers were finished
+         * in the search engine.
+         * If we emit the done-loading signal in this situation the client will think
+         * that it finished the current search, not an old one like it's actually
+         * happening. */
+        if (status == NAUTILUS_SEARCH_PROVIDER_STATUS_NORMAL) {
+                on_search_directory_search_ready_and_valid (search);
+	        nautilus_directory_emit_done_loading (NAUTILUS_DIRECTORY (search));
+        } else if (status == NAUTILUS_SEARCH_PROVIDER_STATUS_RESTARTING) {
+                /* Remove file monitors of the files from an old search that just
+                 * actually finished */
+                reset_file_list (search);
+        }
 }
 
 static void
@@ -605,7 +637,7 @@ search_force_reload (NautilusDirectory *directory)
 		return;
 	}
 	
-	search->details->search_loaded = FALSE;
+	search->details->search_ready_and_valid = FALSE;
 
 	/* Remove file monitors */
 	reset_file_list (search);
@@ -620,7 +652,7 @@ search_are_all_files_seen (NautilusDirectory *directory)
 	search = NAUTILUS_SEARCH_DIRECTORY (directory);
 
 	return (!search->details->query ||
-		search->details->search_loaded);
+		search->details->search_ready_and_valid);
 }
 
 static gboolean
