@@ -166,14 +166,9 @@ action_quit (GSimpleAction *action,
 
 	/* nautilus_window_close() doesn't do anything for desktop windows */
 	windows = nautilus_application_get_windows (application);
-        /* make a copy, since the original list will be modified when destroying
-         * a window, making this list invalid */
-        windows = g_list_copy (windows);
 	for (l = windows; l != NULL; l = l->next) {
 		nautilus_window_close (l->data);
 	}
-
-        g_list_free (windows);
 }
 
 static void
@@ -221,18 +216,6 @@ action_search (GSimpleAction *action,
 	g_object_unref (location);
 }
 
-static void
-action_show_file_transfers (GSimpleAction *action,
-			    GVariant *parameter,
-			    gpointer user_data)
-{
-	NautilusApplication *application = user_data;
-	NautilusProgressUIHandler *progress_handler;
-
-	progress_handler = nautilus_application_get_progress_ui_handler (application);
-	nautilus_progress_ui_handler_ensure_window (progress_handler);
-}
-
 static void
 action_show_hide_sidebar (GSimpleAction *action,
 			  GVariant      *state,
@@ -266,7 +249,6 @@ static GActionEntry app_entries[] = {
 	{ "kill", action_kill, NULL, NULL, NULL },
 	{ "open-desktop", action_open_desktop, NULL, NULL, NULL },
 	{ "close-desktop", action_close_desktop, NULL, NULL, NULL },
-	{ "show-file-transfers", action_show_file_transfers, NULL, NULL, NULL }
 };
 
 void
