@@ -130,12 +130,12 @@ static const SortCriterion sort_criteria[] = {
 	{
 		NAUTILUS_FILE_SORT_BY_MTIME,
 		"modification date",
-		"modification-date"
+		"modification-date",
 	},
 	{
 		NAUTILUS_FILE_SORT_BY_ATIME,
 		"access date",
-		"access-date"
+		"access-date",
 	},
 	{
 		NAUTILUS_FILE_SORT_BY_TRASHED_TIME,
@@ -1047,6 +1047,18 @@ update_sort_action_state_hint (NautilusCanvasView *canvas_view)
 	g_variant_unref (state_hint);
 }
 
+static gboolean
+showing_recent_directory (NautilusView *view)
+{
+	NautilusFile *file;
+
+	file = nautilus_view_get_directory_as_file (view);
+	if (file != NULL) {
+		return nautilus_file_is_in_recent (file);
+	}
+	return FALSE;
+}
+
 static void
 nautilus_canvas_view_update_actions_state (NautilusView *view)
 {
@@ -1095,6 +1107,9 @@ nautilus_canvas_view_update_actions_state (NautilusView *view)
 	action = g_action_map_lookup_action (G_ACTION_MAP (view_action_group), "keep-aligned");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action),
 				     canvas_view->details->supports_keep_aligned);
+	action = g_action_map_lookup_action (G_ACTION_MAP (view_action_group), "sort");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action),
+				     !showing_recent_directory (view));
 	if (canvas_view->details->supports_keep_aligned) {
 		keep_aligned = nautilus_canvas_container_is_keep_aligned (get_canvas_container (canvas_view));
 		g_action_change_state (action, g_variant_new_boolean (keep_aligned));
