@@ -1692,24 +1692,29 @@ location_cell_data_func (GtkTreeViewColumn *column,
 		g_object_unref (query);
 	}
 
-	if (g_file_equal (home_location, dir_location)) {
-		where = g_strdup (_("Home"));
-	} else if (g_file_equal (base_location, dir_location)) {
+	if (g_file_equal (base_location, dir_location)) {
 		/* Only occurs when search result is
 		 * a direct child of the base location
 		 */
 		where = g_strdup ("");
+        } else if (g_file_equal (home_location, dir_location)) {
+		where = g_strdup (_("Home"));
 	} else if (g_file_has_prefix (dir_location, base_location)) {
-	  	where = g_file_get_basename (dir_location);
-	}
+                gchar *relative_path;
 
-	if (where != NULL) {
-		g_object_set (G_OBJECT (renderer),
-		              "text", where,
-		              NULL);
+                relative_path = g_file_get_relative_path (home_location, dir_location);
+                where = g_filename_display_name (relative_path);
 
-		g_free (where);
-	}
+                g_free (relative_path);
+	} else {
+                where =  g_file_get_path (dir_location);
+        }
+
+	g_object_set (G_OBJECT (renderer),
+	              "text", where,
+	              NULL);
+
+	g_free (where);
 
 	g_object_unref (base_location);
 	g_object_unref (dir_location);
@@ -2828,7 +2833,8 @@ create_column_editor (NautilusListView *view)
 	label = gtk_label_new (NULL);
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	gtk_label_set_line_wrap (GTK_LABEL (label), FALSE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+        gtk_label_set_xalign (GTK_LABEL (label), 0);
+        gtk_label_set_yalign (GTK_LABEL (label), 0);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 
@@ -2958,6 +2964,7 @@ nautilus_list_view_click_policy_changed (NautilusView *directory_view)
 	GtkTreeView *tree;
 
 	view = NAUTILUS_LIST_VIEW (directory_view);
+        display = gtk_widget_get_display (GTK_WIDGET (view));
 
 	/* ensure that we unset the hand cursor and refresh underlined rows */
 	if (get_click_policy () == NAUTILUS_CLICK_POLICY_DOUBLE) {
@@ -2977,7 +2984,6 @@ nautilus_list_view_click_policy_changed (NautilusView *directory_view)
 			win = gtk_widget_get_window (GTK_WIDGET (tree));
 			gdk_window_set_cursor (win, NULL);
 
-			display = gtk_widget_get_display (GTK_WIDGET (view));
 			if (display != NULL) {
 				gdk_display_flush (display);
 			}
@@ -2986,7 +2992,7 @@ nautilus_list_view_click_policy_changed (NautilusView *directory_view)
 		g_clear_object (&hand_cursor);
 	} else if (get_click_policy () == NAUTILUS_CLICK_POLICY_SINGLE) {
 		if (hand_cursor == NULL) {
-			hand_cursor = gdk_cursor_new(GDK_HAND2);
+			hand_cursor = gdk_cursor_new_for_display (display, GDK_HAND2);
 		}
 	}
 }
