@@ -68,6 +68,9 @@ struct NautilusWindowSlotDetails {
 	GtkWidget *floating_bar;
 	GtkWidget *view_overlay;
 
+	/* no search results widget */
+	GtkWidget *no_search_results_widget;
+
 	/* slot contains
 	 *  1) an vbox containing extra_location_widgets
 	 *  2) the view
@@ -104,12 +107,9 @@ struct NautilusWindowSlotDetails {
 	gboolean allow_stop;
 	gboolean needs_reload;
 	gboolean load_with_search;
-
-	/* Ensures that we do not react on signals of a
-	 * view that is re-used as new view when its loading
-	 * is cancelled
-	 */
-	gboolean temporarily_ignore_view_signals;
+        /* It could be either the view is loading the files or the search didn't
+         * finish. Used for showing a spinner to provide feedback to the user. */
+        gboolean busy;
 
 	/* New location. */
 	GFile *pending_location;
@@ -124,6 +124,10 @@ struct NautilusWindowSlotDetails {
 	gboolean tried_mount;
 	NautilusWindowGoToCallback open_callback;
 	gpointer open_callback_user_data;
+        gchar *view_mode_before_search;
+
+        /*Folder is empty */
+        GtkWidget *folder_is_empty_widget;
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -132,6 +136,7 @@ static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 static void nautilus_window_slot_force_reload (NautilusWindowSlot *slot);
 static void location_has_really_changed (NautilusWindowSlot *slot);
 static void nautilus_window_slot_connect_new_content_view (NautilusWindowSlot *slot);
+static void nautilus_window_slot_disconnect_content_view (NautilusWindowSlot *slot);
 static void nautilus_window_slot_emit_location_change (NautilusWindowSlot *slot, GFile *from, GFile *to);
 
 static void
@@ -190,6 +195,59 @@ nautilus_window_slot_sync_view_mode (NautilusWindowSlot *slot)
 	}
 }
 
+static void
+remove_loading_floating_bar (NautilusWindowSlot *slot)
+{
+	if (slot->details->loading_timeout_id != 0) {
+		g_source_remove (slot->details->loading_timeout_id);
+		slot->details->loading_timeout_id = 0;
+	}
+
+	gtk_widget_hide (slot->details->floating_bar);
+	nautilus_floating_bar_cleanup_actions (NAUTILUS_FLOATING_BAR (slot->details->floating_bar));
+}
+
+static void
+nautilus_window_slot_on_done_loading (NautilusDirectory  *directory,
+                                      NautilusWindowSlot *slot)
+{
+	GList *files;
+
+	files = nautilus_directory_get_file_list (directory);
+
+        slot->details->busy = FALSE;
+        remove_loading_floating_bar (slot);
+	if (g_list_length (files) != 0) {
+		gtk_widget_hide (slot->details->no_search_results_widget);
+        } else {
+		gtk_widget_show (slot->details->no_search_results_widget);
+        }
+
+	nautilus_file_list_unref (files);
+}
+
+static void
+connect_directory_signals (NautilusWindowSlot *slot,
+                           NautilusDirectory  *directory)
+{
+        if (NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
+                g_signal_connect_object (directory, "done-loading",
+                                         G_CALLBACK (nautilus_window_slot_on_done_loading),
+                                         slot, 0);
+        }
+}
+
+static void
+disconnect_directory_signals (NautilusWindowSlot *slot,
+                              NautilusDirectory  *directory)
+{
+        if (NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
+                g_signal_handlers_disconnect_by_func (directory,
+                                                      G_CALLBACK (nautilus_window_slot_on_done_loading),
+                                                      slot);
+        }
+}
+
 static void
 sync_search_directory (NautilusWindowSlot *slot)
 {
@@ -565,6 +623,7 @@ static void
 nautilus_window_slot_constructed (GObject *object)
 {
 	NautilusWindowSlot *slot = NAUTILUS_WINDOW_SLOT (object);
+        GtkBuilder *builder;
 	GtkWidget *extras_vbox;
 
 	G_OBJECT_CLASS (nautilus_window_slot_parent_class)->constructed (object);
@@ -592,6 +651,16 @@ nautilus_window_slot_constructed (GObject *object)
 	gtk_box_pack_start (GTK_BOX (slot), slot->details->view_overlay, TRUE, TRUE, 0);
 	gtk_widget_show (slot->details->view_overlay);
 
+        builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/nautilus-no-search-results.ui");
+	slot->details->no_search_results_widget = GTK_WIDGET (gtk_builder_get_object (builder, "no_search_results"));
+	gtk_overlay_add_overlay (GTK_OVERLAY (slot->details->view_overlay),
+				 slot->details->no_search_results_widget);
+        builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/nautilus-folder-is-empty.ui");
+	slot->details->folder_is_empty_widget = GTK_WIDGET (gtk_builder_get_object (builder, "folder_is_empty"));
+	gtk_overlay_add_overlay (GTK_OVERLAY (slot->details->view_overlay),
+				 slot->details->folder_is_empty_widget);
+        g_object_unref (builder);
+
 	slot->details->floating_bar = nautilus_floating_bar_new (NULL, NULL, FALSE);
 	gtk_widget_set_halign (slot->details->floating_bar, GTK_ALIGN_END);
 	gtk_widget_set_valign (slot->details->floating_bar, GTK_ALIGN_END);
@@ -611,18 +680,6 @@ nautilus_window_slot_init (NautilusWindowSlot *slot)
 		(slot, NAUTILUS_TYPE_WINDOW_SLOT, NautilusWindowSlotDetails);
 }
 
-static void
-remove_loading_floating_bar (NautilusWindowSlot *slot)
-{
-	if (slot->details->loading_timeout_id != 0) {
-		g_source_remove (slot->details->loading_timeout_id);
-		slot->details->loading_timeout_id = 0;
-	}
-
-	gtk_widget_hide (slot->details->floating_bar);
-	nautilus_floating_bar_cleanup_actions (NAUTILUS_FLOATING_BAR (slot->details->floating_bar));
-}
-
 #define DEBUG_FLAG NAUTILUS_DEBUG_WINDOW
 #include <libnautilus-private/nautilus-debug.h>
 
@@ -859,6 +916,7 @@ begin_location_change (NautilusWindowSlot *slot,
 		       NautilusWindowGoToCallback callback,
 		       gpointer user_data)
 {
+	NautilusDirectory *previous_directory;
         NautilusDirectory *directory;
         NautilusFile *file;
 	gboolean force_reload;
@@ -874,6 +932,22 @@ begin_location_change (NautilusWindowSlot *slot,
 
 	nautilus_profile_start (NULL);
 
+	previous_directory = nautilus_directory_get (previous_location);
+        directory = nautilus_directory_get (location);
+
+        gtk_widget_hide (slot->details->no_search_results_widget);
+	/* Disconnect search signals from the old directory if it was a search directory */
+        disconnect_directory_signals (slot, previous_directory);
+	nautilus_directory_unref (previous_directory);
+
+        /* Avoid to update status from the current view in our async calls */
+        nautilus_window_slot_disconnect_content_view (slot);
+        /* We are going to change the location, so make sure we stop any loading
+         * or searching of the previous view, so we avoid to be slow */
+        if (slot->details->content_view) {
+                nautilus_view_stop_loading (slot->details->content_view);
+        }
+
 	/* If there is no new selection and the new location is
 	 * a (grand)parent of the old location then we automatically
 	 * select the folder the previous location was in */
@@ -915,8 +989,6 @@ begin_location_change (NautilusWindowSlot *slot,
 	slot->details->open_callback = callback;
 	slot->details->open_callback_user_data = user_data;
 
-        directory = nautilus_directory_get (location);
-
 	/* The code to force a reload is here because if we do it
 	 * after determining an initial view (in the components), then
 	 * we end up fetching things twice.
@@ -1140,7 +1212,6 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 {
         GError *error = NULL;
 	char *view_id;
-	char *mimetype;
 	NautilusWindow *window;
 	NautilusWindowSlot *slot;
 	NautilusFile *viewed_file, *parent_file;
@@ -1246,18 +1317,41 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 	    (error->domain == G_IO_ERROR && error->code == G_IO_ERROR_NOT_SUPPORTED)) {
 		/* We got the information we need, now pick what view to use: */
 
-		mimetype = nautilus_file_get_mime_type (file);
+                /* If we are in search, try to use by default list view. This will be deactivated
+                 * if the user manually switch to a diferent view mode */
+                if (nautilus_file_is_in_search (nautilus_file_get (location))) {
+                        if (g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_LIST_VIEW_ON_SEARCH)) {
+                                /* If it's already set, is because we already made the change to search mode,
+                                 * so the view mode of the current view will be the one search is using,
+                                 * which is not the one we are interested in */
+                                if (slot->details->view_mode_before_search == NULL) {
+                                        slot->details->view_mode_before_search = g_strdup (nautilus_view_get_view_id (slot->details->content_view));
+                                }
+                                view_id = g_strdup (NAUTILUS_LIST_VIEW_IID);
+                        } else {
+                                g_free (slot->details->view_mode_before_search);
+                                slot->details->view_mode_before_search = NULL;
+                        }
+                }
 
-		/* Otherwise, use default */
-		if (slot->details->content_view != NULL) {
-			view_id = g_strdup (nautilus_view_get_view_id (slot->details->content_view));
+                /* If there is already a view, just use the view mode that it's currently using, or
+                 * if we were on search before, use what we were using before entering
+                 * search mode */
+		if (slot->details->content_view != NULL && view_id == NULL) {
+                        if (slot->details->view_mode_before_search != NULL) {
+                                view_id = g_strdup (slot->details->view_mode_before_search);
+                                g_free (slot->details->view_mode_before_search);
+                                slot->details->view_mode_before_search = NULL;
+                        } else {
+			        view_id = g_strdup (nautilus_view_get_view_id (slot->details->content_view));
+                        }
 		}
 
+                /* If there is not previous view in this slot, use the default view mode
+                 * from preferences */
 		if (view_id == NULL) {
 			view_id = nautilus_global_preferences_get_default_folder_viewer_preference_as_iid ();
 		}
-
-		g_free (mimetype);
 	}
 
 	if (view_id != NULL) {
@@ -1381,6 +1475,7 @@ create_content_view (NautilusWindowSlot *slot,
 		view_id = NAUTILUS_DESKTOP_ICON_VIEW_IID;
 	}
 
+        nautilus_window_slot_disconnect_content_view (slot);
         if (nautilus_window_slot_content_view_matches_iid (slot, view_id)) {
                 /* reuse existing content view */
                 view = slot->details->content_view;
@@ -1391,14 +1486,18 @@ create_content_view (NautilusWindowSlot *slot,
 		view = nautilus_view_new (view_id, slot);
 
                 slot->details->new_content_view = view;
-		nautilus_window_slot_connect_new_content_view (slot);
         }
+        nautilus_window_slot_connect_new_content_view (slot);
 
 	/* Forward search selection and state before loading the new model */
 	old_location = nautilus_window_slot_get_location (slot);
 	old_directory = nautilus_directory_get (old_location);
 	new_directory = nautilus_directory_get (slot->details->pending_location);
 
+        /* Connect to the done loading signal if it is a search directory to update
+         * the no results widget */
+        connect_directory_signals (slot, new_directory);
+
 	if (NAUTILUS_IS_SEARCH_DIRECTORY (new_directory) &&
 	    !NAUTILUS_IS_SEARCH_DIRECTORY (old_directory)) {
 		nautilus_search_directory_set_base_model (NAUTILUS_SEARCH_DIRECTORY (new_directory), old_directory);
@@ -1543,10 +1642,6 @@ free_location_change (NautilusWindowSlot *slot)
         }
 
         if (slot->details->new_content_view != NULL) {
-		slot->details->temporarily_ignore_view_signals = TRUE;
-		nautilus_view_stop_loading (slot->details->new_content_view);
-		slot->details->temporarily_ignore_view_signals = FALSE;
-
 		g_object_unref (slot->details->new_content_view);
 		slot->details->new_content_view = NULL;
         }
@@ -1557,9 +1652,16 @@ cancel_location_change (NautilusWindowSlot *slot)
 {
 	GList *selection;
 	GFile *location;
+        NautilusDirectory *directory;
 
 	location = nautilus_window_slot_get_location (slot);
 
+	directory = nautilus_directory_get (slot->details->location);
+        disconnect_directory_signals (slot, directory);
+        /* Stops current loading or search if any, so we are not slow */
+	nautilus_view_stop_loading (slot->details->content_view);
+        nautilus_directory_unref (directory);
+
         if (slot->details->pending_location != NULL
             && location != NULL
             && slot->details->content_view != NULL) {
@@ -1743,6 +1845,7 @@ nautilus_window_slot_force_reload (NautilusWindowSlot *slot)
 
 	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
 
+        gtk_widget_hide (slot->details->no_search_results_widget);
 	location = nautilus_window_slot_get_location (slot);
 	if (location == NULL) {
 		return;
@@ -2148,9 +2251,9 @@ view_end_loading_cb (NautilusView       *view,
 		     gboolean            all_files_seen,
 		     NautilusWindowSlot *slot)
 {
-	if (slot->details->temporarily_ignore_view_signals) {
-		return;
-	}
+        NautilusDirectory *directory;
+        GList *files;
+        gboolean show_folder_is_empty;
 
 	/* Only handle this if we're expecting it.
 	 * Don't handle it if its from an old view we've switched from */
@@ -2167,7 +2270,25 @@ view_end_loading_cb (NautilusView       *view,
 		slot->details->needs_reload = FALSE;
 	}
 
-	remove_loading_floating_bar (slot);
+        directory = nautilus_view_get_model (slot->details->content_view);
+        /* If there is no directory associated, we are probably in the middle
+         * of some change, so better to not show anything so it doesn't flash */
+        if (directory != NULL) {
+                files = nautilus_directory_get_file_list (directory);
+                show_folder_is_empty = g_list_length (files) == 0;
+                nautilus_file_list_free (files);
+        } else {
+                show_folder_is_empty = FALSE;
+        }
+        /* If it is a search directory, it will hide the toolbar when the search engine
+         * finishes, not every time the view end loading the new files */
+        if (!NAUTILUS_IS_SEARCH_DIRECTORY (nautilus_view_get_model (slot->details->content_view))) {
+                slot->details->busy = FALSE;
+                if (show_folder_is_empty) {
+                        gtk_widget_show (slot->details->folder_is_empty_widget);
+                }
+	        remove_loading_floating_bar (slot);
+        }
 }
 
 static void
@@ -2190,7 +2311,7 @@ real_setup_loading_floating_bar (NautilusWindowSlot *slot)
 						 _("Searching…") : _("Loading…"));
 	nautilus_floating_bar_set_details_label (NAUTILUS_FLOATING_BAR (slot->details->floating_bar), NULL);
 	nautilus_floating_bar_set_show_spinner (NAUTILUS_FLOATING_BAR (slot->details->floating_bar),
-						TRUE);
+						slot->details->busy);
 	nautilus_floating_bar_add_action (NAUTILUS_FLOATING_BAR (slot->details->floating_bar),
 					  "process-stop-symbolic",
 					  NAUTILUS_FLOATING_BAR_ACTION_ID_STOP);
@@ -2232,12 +2353,10 @@ static void
 view_begin_loading_cb (NautilusView       *view,
 		       NautilusWindowSlot *slot)
 {
-	if (slot->details->temporarily_ignore_view_signals) {
-		return;
-	}
-
 	nautilus_profile_start (NULL);
 
+        slot->details->busy = TRUE;
+        gtk_widget_hide (slot->details->folder_is_empty_widget);
 	if (view == slot->details->new_content_view) {
 		location_has_really_changed (slot);
 	} else {
@@ -2312,17 +2431,21 @@ nautilus_window_slot_setup_extra_location_widgets (NautilusWindowSlot *slot)
 
 static void
 nautilus_window_slot_connect_new_content_view (NautilusWindowSlot *slot)
+{
+	if (slot->details->new_content_view != NULL) {
+		g_signal_connect (slot->details->new_content_view, "begin-loading", G_CALLBACK (view_begin_loading_cb), slot);
+		g_signal_connect (slot->details->new_content_view, "end-loading", G_CALLBACK (view_end_loading_cb), slot);
+	}
+}
+
+static void
+nautilus_window_slot_disconnect_content_view (NautilusWindowSlot *slot)
 {
 	if (slot->details->content_view != NULL) {
 		/* disconnect old view */
 		g_signal_handlers_disconnect_by_func (slot->details->content_view, G_CALLBACK (view_end_loading_cb), slot);
 		g_signal_handlers_disconnect_by_func (slot->details->content_view, G_CALLBACK (view_begin_loading_cb), slot);
 	}
-
-	if (slot->details->new_content_view != NULL) {
-		g_signal_connect (slot->details->new_content_view, "begin-loading", G_CALLBACK (view_begin_loading_cb), slot);
-		g_signal_connect (slot->details->new_content_view, "end-loading", G_CALLBACK (view_end_loading_cb), slot);
-	}
 }
 
 static void
@@ -2401,6 +2524,7 @@ static void
 nautilus_window_slot_dispose (GObject *object)
 {
 	NautilusWindowSlot *slot;
+	NautilusDirectory *directory;
 	GtkWidget *widget;
 
 	slot = NAUTILUS_WINDOW_SLOT (object);
@@ -2443,8 +2567,17 @@ nautilus_window_slot_dispose (GObject *object)
 		/* TODO? why do we ref here, instead of unreffing?
 		 * It was already here before the slot migration, though */
 		g_object_ref (slot->details->location);
+
+		directory = nautilus_directory_get (slot->details->location);
+                disconnect_directory_signals (slot, directory);
+                g_object_unref (directory);
 	}
 
+        if (slot->details->view_mode_before_search) {
+                g_free (slot->details->view_mode_before_search);
+                slot->details->view_mode_before_search = NULL;
+        }
+
 	g_list_free_full (slot->details->pending_selection, g_object_unref);
 	slot->details->pending_selection = NULL;
 
@@ -2630,13 +2763,6 @@ nautilus_window_slot_set_allow_stop (NautilusWindowSlot *slot,
 void
 nautilus_window_slot_stop_loading (NautilusWindowSlot *slot)
 {
-	nautilus_view_stop_loading (slot->details->content_view);
-
-	if (slot->details->new_content_view != NULL) {
-		slot->details->temporarily_ignore_view_signals = TRUE;
-		nautilus_view_stop_loading (slot->details->new_content_view);
-		slot->details->temporarily_ignore_view_signals = FALSE;
-	}
 
         cancel_location_change (slot);
 }
@@ -2650,7 +2776,7 @@ real_slot_set_short_status (NautilusWindowSlot *slot,
 
 	nautilus_floating_bar_cleanup_actions (NAUTILUS_FLOATING_BAR (slot->details->floating_bar));
 	nautilus_floating_bar_set_show_spinner (NAUTILUS_FLOATING_BAR (slot->details->floating_bar),
-						FALSE);
+						slot->details->busy);
 
 	g_object_get (nautilus_window_slot_get_window (slot),
 		      "disable-chrome", &disable_chrome,
