@@ -110,6 +110,7 @@ struct _NautilusWindowPrivate {
         int side_pane_width;
         GtkWidget *sidebar;        /* container for the GtkPlacesSidebar */
         GtkWidget *places_sidebar; /* the actual GtkPlacesSidebar */
+        GVolume *selected_volume; /* the selected volume in the sidebar popup callback */
 
         /* Main view */
         GtkWidget *main_view;
@@ -120,6 +121,12 @@ struct _NautilusWindowPrivate {
         GtkWidget *notification_delete_close;
         GtkWidget *notification_delete_undo;
 	guint notification_delete_timeout_id;
+        GtkWidget *notification_operation;
+        GtkWidget *notification_operation_label;
+        GtkWidget *notification_operation_close;
+        GtkWidget *notification_operation_open;
+	guint notification_operation_timeout_id;
+        GFile *folder_to_open;
 
         /* Toolbar */
         GtkWidget *toolbar;
@@ -439,14 +446,36 @@ action_view_mode (GSimpleAction *action,
 {
 	const gchar *name;
 	NautilusWindowSlot *slot;
+        GFile *location;
 
 	name =  g_variant_get_string (value, NULL);
 	slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
 
 	if (g_strcmp0 (name, "list") == 0) {
 		nautilus_window_slot_set_content_view (slot, NAUTILUS_LIST_VIEW_ID);
+                /* If this change is caused because of the automatic list view
+                 * switch for search, don't set as default list view */
+                location = nautilus_window_slot_get_location (slot);
+                if (!(nautilus_file_is_in_search (nautilus_file_get (location))  &&
+                      g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_LIST_VIEW_ON_SEARCH))) {;
+                        g_settings_set_enum (nautilus_preferences,
+                                             NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER,
+                                             NAUTILUS_DEFAULT_FOLDER_VIEWER_LIST_VIEW);
+                }
 	} else if (g_strcmp0 (name, "grid") == 0) {
 		nautilus_window_slot_set_content_view (slot, NAUTILUS_CANVAS_VIEW_ID);
+
+                /* If the user manually changed the view mode to grid, disable the automatic
+                 * switch to list view on search */
+                location = nautilus_window_slot_get_location (slot);
+                if (nautilus_file_is_in_search (nautilus_file_get (location))) {
+                      g_settings_set_boolean (nautilus_preferences,
+                                              NAUTILUS_PREFERENCES_LIST_VIEW_ON_SEARCH,
+                                              FALSE);
+                }
+                g_settings_set_enum (nautilus_preferences,
+                                     NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER,
+                                     NAUTILUS_DEFAULT_FOLDER_VIEWER_ICON_VIEW);
 	} else {
 		g_assert_not_reached ();
 	}
@@ -503,97 +532,6 @@ undo_manager_changed (NautilusWindow *window)
 	g_free (redo_description);
 }
 
-const GActionEntry win_entries[] = {
-	{ "back",  action_back },
-	{ "forward",  action_forward },
-	{ "up",  action_up },
-	{ "action-menu", action_toggle_state_action_button, NULL, "false", NULL },
-	{ "reload", action_reload },
-	{ "stop", action_stop },
-	{ "new-tab", action_new_tab },
-	{ "enter-location", action_enter_location },
-	{ "bookmark-current-location", action_bookmark_current_location },
-	{ "toggle-search", NULL, NULL, "false", action_toggle_search },
-	{ "undo", action_undo },
-	{ "redo", action_redo },
-	{ "view-mode", NULL, "s", "''", action_view_mode },
-	/* Only accesible by shorcuts */
-	{ "close-current-view", action_close_current_view },
-	{ "go-home", action_go_home },
-	{ "tab-previous", action_tab_previous },
-	{ "tab-next", action_tab_next },
-	{ "tab-move-left", action_tab_move_left },
-	{ "tab-move-right", action_tab_move_right },
-	{ "prompt-root-location", action_prompt_for_location_root },
-	{ "prompt-home-location", action_prompt_for_location_home },
-	{ "go-to-tab", NULL, "i", "0", action_go_to_tab },
-};
-
-static void
-nautilus_window_initialize_actions (NautilusWindow *window)
-{
-	GApplication *app;
-	GAction *action;
-	GVariant *state;
-	gchar detailed_action[80];
-	gchar accel[80];
-	gint i;
-	const gchar *reload_accels[] = {
-                "F5",
-                "<ctrl>r",
-                NULL
-        };
-
-	g_action_map_add_action_entries (G_ACTION_MAP (window),
-					 win_entries, G_N_ELEMENTS (win_entries),
-					 window);
-
-	app = g_application_get_default ();
-	nautilus_application_add_accelerator (app, "win.back", "<alt>Left");
-	nautilus_application_add_accelerator (app, "win.forward", "<alt>Right");
-	nautilus_application_add_accelerator (app, "win.enter-location", "<control>l");
-	nautilus_application_add_accelerator (app, "win.new-tab", "<control>t");
-	nautilus_application_add_accelerator (app, "win.toggle-search", "<control>f");
-	nautilus_application_add_accelerator (app, "win.view-mode('list')", "<control>1");
-	nautilus_application_add_accelerator (app, "win.view-mode('grid')", "<control>2");
-	nautilus_application_add_accelerator (app, "win.close-current-view", "<control>w");
-
-        /* Special case reload, since users are used to use two shortcuts instead of one */
-	gtk_application_set_accels_for_action (GTK_APPLICATION (app), "win.reload", reload_accels);
-
-	nautilus_application_add_accelerator (app, "win.undo", "<control>z");
-	nautilus_application_add_accelerator (app, "win.redo", "<shift><control>z");
-	/* Only accesible by shorcuts */
-	nautilus_application_add_accelerator (app, "win.bookmark-current-location", "<control>d");
-	nautilus_application_add_accelerator (app, "win.up", "<alt>Up");
-	nautilus_application_add_accelerator (app, "win.go-home", "<alt>Home");
-	nautilus_application_add_accelerator (app, "win.tab-previous", "<control>Page_Up");
-	nautilus_application_add_accelerator (app, "win.tab-next", "<control>Page_Down");
-	nautilus_application_add_accelerator (app, "win.tab-move-left", "<shift><control>Page_Up");
-	nautilus_application_add_accelerator (app, "win.tab-move-right", "<shift><control>Page_Down");
-	nautilus_application_add_accelerator (app, "win.prompt-root-location", "slash");
-	nautilus_application_add_accelerator (app, "win.prompt-home-location", "asciitilde");
-	nautilus_application_add_accelerator (app, "win.action-menu", "F10");
-
-	/* Alt+N for the first 9 tabs */
-	for (i = 0; i < 9; ++i) {
-		g_snprintf (detailed_action, sizeof (detailed_action), "win.go-to-tab(%i)", i);
-		g_snprintf (accel, sizeof (accel), "<alt>%i", i + 1);
-		nautilus_application_add_accelerator (app, detailed_action, accel);
-	}
-
-	action = g_action_map_lookup_action (G_ACTION_MAP (app), "show-hide-sidebar");
-	state = g_action_get_state (action);
-	if (g_variant_get_boolean (state))
-		nautilus_window_show_sidebar (window);
-
-	g_variant_unref (state);
-
-	g_signal_connect_object (nautilus_file_undo_manager_get (), "undo-changed",
-				 G_CALLBACK (undo_manager_changed), window, G_CONNECT_SWAPPED);
-	undo_manager_changed (window);
-}
-
 void
 nautilus_window_go_to (NautilusWindow *window, GFile *location)
 {
@@ -823,7 +761,10 @@ update_cursor (NautilusWindow *window)
 	slot = nautilus_window_get_active_slot (window);
 
 	if (nautilus_window_slot_get_allow_stop (slot)) {
-		cursor = gdk_cursor_new (GDK_WATCH);
+                GdkDisplay *display;
+
+                display = gtk_widget_get_display (GTK_WIDGET (window));
+		cursor = gdk_cursor_new_for_display (display, GDK_WATCH);
 		gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), cursor);
 		g_object_unref (cursor);
 	} else {
@@ -1169,8 +1110,9 @@ window_loading_uri_cb (NautilusWindow *window,
 
 /* Callback used in the "empty trash" menu item from the places sidebar */
 static void
-empty_trash_cb (GtkMenuItem *item,
-		gpointer     user_data)
+action_empty_trash (GSimpleAction *action,
+                    GVariant      *variant,
+                    gpointer       user_data)
 {
 	NautilusWindow *window = NAUTILUS_WINDOW (user_data);
 
@@ -1179,8 +1121,9 @@ empty_trash_cb (GtkMenuItem *item,
 
 /* Callback used for the "properties" menu item from the places sidebar */
 static void
-properties_cb (GtkMenuItem *item,
-	       gpointer     user_data)
+action_properties (GSimpleAction *action,
+                   GVariant      *variant,
+                   gpointer       user_data)
 {
 	NautilusWindow *window = NAUTILUS_WINDOW (user_data);
 	GFile *selected;
@@ -1223,20 +1166,17 @@ should_show_format_command (GVolume *volume)
 }
 
 static void
-format_cb (GtkMenuItem *item,
-	   gpointer     user_data)
+action_format (GSimpleAction *action,
+               GVariant      *variant,
+               gpointer       user_data)
 {
 	NautilusWindow *window = NAUTILUS_WINDOW (user_data);
 	GAppInfo *app_info;
 	gchar *cmdline, *device_identifier, *xid_string;
-	GVolume *volume;
 	gint xid;
 
-	volume = g_object_get_data (G_OBJECT (item), "nautilus-volume");
-
-	g_assert (volume != NULL && G_IS_VOLUME (volume));
-
-	device_identifier = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	device_identifier = g_volume_get_identifier (window->priv->selected_volume,
+                                              G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	xid = (gint) gdk_x11_window_get_xid (gtk_widget_get_window (GTK_WIDGET (window)));
 	xid_string = g_strdup_printf ("%d", xid);
 
@@ -1252,65 +1192,80 @@ format_cb (GtkMenuItem *item,
 	g_free (device_identifier);
 	g_free (xid_string);
 	g_clear_object (&app_info);
+	g_clear_object (&window->priv->selected_volume);
 }
 
-/* Destroy notification function used from g_object_set_data_full() */
 static void
-menu_item_destroy_notify_cb (gpointer data)
+add_menu_separator (GtkWidget *menu)
 {
-	GVolume *volume;
+        GtkWidget *separator;
 
-	volume = G_VOLUME (data);
-	g_object_unref (volume);
+        separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+        gtk_container_add (GTK_CONTAINER (menu), separator);
+        gtk_widget_set_margin_top (separator, 6);
+        gtk_widget_set_margin_bottom (separator, 6);
+        gtk_widget_show (separator);
 }
 
 static void
 places_sidebar_populate_popup_cb (GtkPlacesSidebar *sidebar,
-				  GtkMenu          *menu,
+				  GtkWidget        *menu,
 				  GFile            *selected_item,
 				  GVolume          *selected_volume,
 				  gpointer          user_data)
 {
 	NautilusWindow *window = NAUTILUS_WINDOW (user_data);
-	GtkWidget *item;
 	GFile *trash;
+        GtkWidget *menu_item;
+        GAction *action;
 
 	if (selected_item) {
 		trash = g_file_new_for_uri ("trash:///");
 		if (g_file_equal (trash, selected_item)) {
-			eel_gtk_menu_append_separator (menu);
+                        add_menu_separator (menu);
 
-			item = gtk_menu_item_new_with_mnemonic (_("Empty _Trash"));
-			g_signal_connect (item, "activate",
-					  G_CALLBACK (empty_trash_cb), window);
-			gtk_widget_show (item);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+                        menu_item = gtk_model_button_new ();
+                        gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_item),
+                                                        "win.empty-trash");
+                        g_object_set (menu_item, "text", _("Empty _Trash"), NULL);
+                        gtk_container_add (GTK_CONTAINER (menu), menu_item);
+                        gtk_widget_show (menu_item);
 
-			if (nautilus_trash_monitor_is_empty ())
-				gtk_widget_set_sensitive (item, FALSE);
+                        action = g_action_map_lookup_action (G_ACTION_MAP (window),
+                                                             "empty-trash");
+                        g_simple_action_set_enabled (G_SIMPLE_ACTION (action),
+                                                     !nautilus_trash_monitor_is_empty ());
 		}
 		g_object_unref (trash);
 
 		if (g_file_is_native (selected_item)) {
-			eel_gtk_menu_append_separator (menu);
+                        add_menu_separator (menu);
 
-			item = gtk_menu_item_new_with_mnemonic (_("_Properties"));
-			g_signal_connect (item, "activate",
-					  G_CALLBACK (properties_cb), window);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-			gtk_widget_show (item);
+                        menu_item = gtk_model_button_new ();
+                        gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_item),
+                                                        "win.properties");
+                        g_object_set (menu_item, "text", _("_Properties"), NULL);
+                        gtk_container_add (GTK_CONTAINER (menu), menu_item);
+                        gtk_widget_show (menu_item);
 		}
 	}
 	if (selected_volume) {
 		if (should_show_format_command (selected_volume)) {
-			item = gtk_menu_item_new_with_mnemonic (_("_Format…"));
-			g_object_set_data_full (G_OBJECT (item), "nautilus-volume",
-						g_object_ref (selected_volume),
-						menu_item_destroy_notify_cb);
-			g_signal_connect (item, "activate",
-					  G_CALLBACK (format_cb), window);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-			gtk_widget_show (item);
+                        menu_item = gtk_model_button_new ();
+                        gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_item),
+                                                        "win.format");
+                        g_object_set (menu_item, "text", _("_Format…"), NULL);
+                        if (selected_volume != NULL && G_IS_VOLUME (selected_volume)) {
+                                window->priv->selected_volume = g_object_ref (selected_volume);
+                        }
+                        gtk_container_add (GTK_CONTAINER (menu), menu_item);
+                        gtk_widget_show (menu_item);
+
+                        action = g_action_map_lookup_action (G_ACTION_MAP (window),
+                                                             "format");
+                        g_simple_action_set_enabled (G_SIMPLE_ACTION (action),
+                                                     selected_volume != NULL &&
+                                                     G_IS_VOLUME (selected_volume));
 		}
 	}
 }
@@ -1482,51 +1437,67 @@ nautilus_window_ensure_location_entry (NautilusWindow *window)
 }
 
 static void
-nautilus_window_on_notification_delete_undo_clicked (GtkWidget  *notification,
-                                                     gpointer    user_data)
+remove_notifications (NautilusWindow *window)
 {
-	NautilusWindow *window;
+        GtkRevealerTransitionType transition_type;
 
-	window = NAUTILUS_WINDOW (user_data);
+        /* Hide it inmediatily so we can animate the new notification. */
+        transition_type = gtk_revealer_get_transition_type (GTK_REVEALER (window->priv->notification_delete));
+        gtk_revealer_set_transition_type (GTK_REVEALER (window->priv->notification_delete),
+                                          GTK_REVEALER_TRANSITION_TYPE_NONE);
+        gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_delete),
+                                       FALSE);
+        gtk_revealer_set_transition_type (GTK_REVEALER (window->priv->notification_delete),
+                                          transition_type);
+        if (window->priv->notification_delete_timeout_id != 0) {
+                g_source_remove (window->priv->notification_delete_timeout_id);
+                window->priv->notification_delete_timeout_id = 0;
+	}
 
+        transition_type = gtk_revealer_get_transition_type (GTK_REVEALER (window->priv->notification_operation));
+        gtk_revealer_set_transition_type (GTK_REVEALER (window->priv->notification_operation),
+                                          GTK_REVEALER_TRANSITION_TYPE_NONE);
+        gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_operation),
+                                       FALSE);
+        gtk_revealer_set_transition_type (GTK_REVEALER (window->priv->notification_operation),
+                                          transition_type);
+        if (window->priv->notification_operation_timeout_id != 0) {
+                g_source_remove (window->priv->notification_operation_timeout_id);
+                window->priv->notification_operation_timeout_id = 0;
+	}
+}
+
+static void
+hide_notification_delete (NautilusWindow *window)
+{
 	if (window->priv->notification_delete_timeout_id != 0) {
 		g_source_remove (window->priv->notification_delete_timeout_id);
 		window->priv->notification_delete_timeout_id = 0;
 	}
 
 	gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_delete), FALSE);
+}
+
+static void
+nautilus_window_on_notification_delete_undo_clicked (GtkWidget      *notification,
+                                                     NautilusWindow *window)
+{
+        hide_notification_delete (window);
+
 	nautilus_file_undo_manager_undo (GTK_WINDOW (window));
 }
 
 static void
-nautilus_window_on_notification_delete_close_clicked (GtkWidget  *notification,
-                                                      gpointer    user_data)
+nautilus_window_on_notification_delete_close_clicked (GtkWidget      *notification,
+                                                      NautilusWindow *window)
 {
-	NautilusWindow *window;
-
-	window = NAUTILUS_WINDOW (user_data);
-
-	if (window->priv->notification_delete_timeout_id != 0) {
-		g_source_remove (window->priv->notification_delete_timeout_id);
-		window->priv->notification_delete_timeout_id = 0;
-	}
-
-	gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_delete), FALSE);
+        hide_notification_delete (window);
 }
 
 static gboolean
-nautilus_window_on_notification_delete_timeout (gpointer user_data)
+nautilus_window_on_notification_delete_timeout (NautilusWindow *window)
 {
-	NautilusWindow *window;
-
-	window = NAUTILUS_WINDOW (user_data);
-
-	if (window->priv->notification_delete_timeout_id != 0) {
-		g_source_remove (window->priv->notification_delete_timeout_id);
-		window->priv->notification_delete_timeout_id = 0;
-	}
-
-	gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_delete), FALSE);
+        hide_notification_delete (window);
 
 	return FALSE;
 }
@@ -1560,20 +1531,9 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
 {
 	NautilusFileUndoInfo *undo_info;
 	NautilusFileUndoManagerState state;
-	int transition_durantion;
 	gchar *label;
 	GList *files;
 
-	/* Hide it inmediatily so we can animate the new notification. */
-	transition_durantion = gtk_revealer_get_transition_duration (GTK_REVEALER (window->priv->notification_delete));
-	gtk_revealer_set_transition_duration (GTK_REVEALER (window->priv->notification_delete), 0);
-	gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_delete), FALSE);
-	gtk_revealer_set_transition_duration (GTK_REVEALER (window->priv->notification_delete), transition_durantion);
-	if (window->priv->notification_delete_timeout_id != 0) {
-		g_source_remove (window->priv->notification_delete_timeout_id);
-		window->priv->notification_delete_timeout_id = 0;
-	}
-
 	undo_info = nautilus_file_undo_manager_get_action ();
 	state = nautilus_file_undo_manager_get_state ();
 
@@ -1591,7 +1551,7 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
 			gtk_label_set_markup (GTK_LABEL (window->priv->notification_delete_label), label);
 			gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_delete), TRUE);
 			window->priv->notification_delete_timeout_id = g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
-			                                                                      nautilus_window_on_notification_delete_timeout,
+			                                                                      (GSourceFunc) nautilus_window_on_notification_delete_timeout,
 			                                                                      window);
 			g_free (label);
 		}
@@ -1599,6 +1559,82 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
 	}
 }
 
+static void
+hide_notification_operation (NautilusWindow *window)
+{
+	if (window->priv->notification_operation_timeout_id != 0) {
+		g_source_remove (window->priv->notification_operation_timeout_id);
+		window->priv->notification_operation_timeout_id = 0;
+	}
+
+	gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_operation), FALSE);
+        g_clear_object (&window->priv->folder_to_open);
+}
+
+static void
+on_notification_operation_open_clicked (GtkWidget      *notification,
+                                        NautilusWindow *window)
+{
+        nautilus_window_slot_open_location (window->priv->active_slot,
+                                            window->priv->folder_to_open,
+                                            0);
+        hide_notification_operation (window);
+}
+
+static void
+on_notification_operation_close_clicked (GtkWidget      *notification,
+                                         NautilusWindow *window)
+{
+        hide_notification_operation (window);
+}
+
+static gboolean
+on_notification_operation_timeout (NautilusWindow *window)
+{
+        hide_notification_operation (window);
+
+	return FALSE;
+}
+
+void
+nautilus_window_show_operation_notification (NautilusWindow *window,
+                                             gchar          *main_label,
+                                             GFile          *folder_to_open)
+{
+        gchar *button_label;
+        gchar *folder_name;
+        NautilusFile *folder;
+        GFile *current_location;
+
+        current_location = nautilus_window_slot_get_location (window->priv->active_slot);
+	if (gtk_window_has_toplevel_focus (GTK_WINDOW (window)) &&
+            !NAUTILUS_IS_DESKTOP_WINDOW (window)) {
+                remove_notifications (window);
+	        gtk_label_set_text (GTK_LABEL (window->priv->notification_operation_label),
+                                    main_label);
+
+                if (g_file_equal (folder_to_open, current_location)) {
+                        gtk_widget_hide (window->priv->notification_operation_open);
+                } else {
+                        gtk_widget_show (window->priv->notification_operation_open);
+                        window->priv->folder_to_open = g_object_ref (folder_to_open);
+                        folder = nautilus_file_get (folder_to_open);
+                        folder_name = nautilus_file_get_display_name (folder);
+                        button_label = g_strdup_printf (_("Open %s"), folder_name);
+                        gtk_button_set_label (GTK_BUTTON (window->priv->notification_operation_open),
+                                              button_label);
+                        g_object_unref (folder);
+                        g_free (folder_name);
+                        g_free (button_label);
+                }
+
+	        gtk_revealer_set_reveal_child (GTK_REVEALER (window->priv->notification_operation), TRUE);
+	        window->priv->notification_operation_timeout_id = g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
+                                                                                         (GSourceFunc) on_notification_operation_timeout,
+                                                                                         window);
+	}
+}
+
 static void
 path_bar_location_changed_callback (GtkWidget      *widget,
 				    GFile          *location,
@@ -1951,6 +1987,101 @@ setup_notebook (NautilusWindow *window)
 				window);
 }
 
+const GActionEntry win_entries[] = {
+	{ "back",  action_back },
+	{ "forward",  action_forward },
+	{ "up",  action_up },
+	{ "action-menu", action_toggle_state_action_button, NULL, "false", NULL },
+	{ "reload", action_reload },
+	{ "stop", action_stop },
+	{ "new-tab", action_new_tab },
+	{ "enter-location", action_enter_location },
+	{ "bookmark-current-location", action_bookmark_current_location },
+	{ "toggle-search", NULL, NULL, "false", action_toggle_search },
+	{ "undo", action_undo },
+	{ "redo", action_redo },
+	{ "view-mode", NULL, "s", "''", action_view_mode },
+	/* Only accesible by shorcuts */
+	{ "close-current-view", action_close_current_view },
+	{ "go-home", action_go_home },
+	{ "tab-previous", action_tab_previous },
+	{ "tab-next", action_tab_next },
+	{ "tab-move-left", action_tab_move_left },
+	{ "tab-move-right", action_tab_move_right },
+	{ "prompt-root-location", action_prompt_for_location_root },
+	{ "prompt-home-location", action_prompt_for_location_home },
+	{ "go-to-tab", NULL, "i", "0", action_go_to_tab },
+	{ "empty-trash", action_empty_trash },
+	{ "properties", action_properties },
+	{ "format", action_format },
+};
+
+static void
+nautilus_window_initialize_actions (NautilusWindow *window)
+{
+	GApplication *app;
+	GAction *action;
+	GVariant *state;
+	gchar detailed_action[80];
+	gchar accel[80];
+	gint i;
+	const gchar *reload_accels[] = {
+                "F5",
+                "<ctrl>r",
+                NULL
+        };
+
+	g_action_map_add_action_entries (G_ACTION_MAP (window),
+					 win_entries, G_N_ELEMENTS (win_entries),
+					 window);
+
+	app = g_application_get_default ();
+	nautilus_application_add_accelerator (app, "win.back", "<alt>Left");
+	nautilus_application_add_accelerator (app, "win.forward", "<alt>Right");
+	nautilus_application_add_accelerator (app, "win.enter-location", "<control>l");
+	nautilus_application_add_accelerator (app, "win.new-tab", "<control>t");
+	nautilus_application_add_accelerator (app, "win.toggle-search", "<control>f");
+	nautilus_application_add_accelerator (app, "win.view-mode('list')", "<control>1");
+	nautilus_application_add_accelerator (app, "win.view-mode('grid')", "<control>2");
+	nautilus_application_add_accelerator (app, "win.close-current-view", "<control>w");
+
+        /* Special case reload, since users are used to use two shortcuts instead of one */
+	gtk_application_set_accels_for_action (GTK_APPLICATION (app), "win.reload", reload_accels);
+
+	nautilus_application_add_accelerator (app, "win.undo", "<control>z");
+	nautilus_application_add_accelerator (app, "win.redo", "<shift><control>z");
+	/* Only accesible by shorcuts */
+	nautilus_application_add_accelerator (app, "win.bookmark-current-location", "<control>d");
+	nautilus_application_add_accelerator (app, "win.up", "<alt>Up");
+	nautilus_application_add_accelerator (app, "win.go-home", "<alt>Home");
+	nautilus_application_add_accelerator (app, "win.tab-previous", "<control>Page_Up");
+	nautilus_application_add_accelerator (app, "win.tab-next", "<control>Page_Down");
+	nautilus_application_add_accelerator (app, "win.tab-move-left", "<shift><control>Page_Up");
+	nautilus_application_add_accelerator (app, "win.tab-move-right", "<shift><control>Page_Down");
+	nautilus_application_add_accelerator (app, "win.prompt-root-location", "slash");
+	nautilus_application_add_accelerator (app, "win.prompt-home-location", "asciitilde");
+	nautilus_application_add_accelerator (app, "win.action-menu", "F10");
+
+	/* Alt+N for the first 9 tabs */
+	for (i = 0; i < 9; ++i) {
+		g_snprintf (detailed_action, sizeof (detailed_action), "win.go-to-tab(%i)", i);
+		g_snprintf (accel, sizeof (accel), "<alt>%i", i + 1);
+		nautilus_application_add_accelerator (app, detailed_action, accel);
+	}
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (app), "show-hide-sidebar");
+	state = g_action_get_state (action);
+	if (g_variant_get_boolean (state))
+		nautilus_window_show_sidebar (window);
+
+	g_variant_unref (state);
+
+	g_signal_connect_object (nautilus_file_undo_manager_get (), "undo-changed",
+				 G_CALLBACK (undo_manager_changed), window, G_CONNECT_SWAPPED);
+	undo_manager_changed (window);
+}
+
+
 static void
 nautilus_window_constructed (GObject *self)
 {
@@ -2098,9 +2229,14 @@ nautilus_window_finalize (GObject *object)
 		window->priv->sidebar_width_handler_id = 0;
 	}
 
-	if (window->priv->notification_delete_timeout_id != 0) {
-		g_source_remove (window->priv->notification_delete_timeout_id);
-		window->priv->notification_delete_timeout_id = 0;
+        if (window->priv->notification_delete_timeout_id != 0) {
+                g_source_remove (window->priv->notification_delete_timeout_id);
+                window->priv->notification_delete_timeout_id = 0;
+	}
+
+        if (window->priv->notification_operation_timeout_id != 0) {
+                g_source_remove (window->priv->notification_operation_timeout_id);
+                window->priv->notification_operation_timeout_id = 0;
 	}
 
 	g_signal_handlers_disconnect_by_func (nautilus_file_undo_manager_get (),
@@ -2502,6 +2638,10 @@ nautilus_window_class_init (NautilusWindowClass *class)
 	gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_delete_label);
 	gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_delete_undo);
 	gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_delete_close);
+	gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_operation);
+	gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_operation_label);
+	gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_operation_open);
+	gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_operation_close);
 
 	properties[PROP_DISABLE_CHROME] =
 		g_param_spec_boolean ("disable-chrome",
@@ -2552,6 +2692,9 @@ nautilus_window_class_init (NautilusWindowClass *class)
 				  G_CALLBACK(use_extra_mouse_buttons_changed),
 				  NULL);
 
+        gtk_widget_class_bind_template_callback (wclass, on_notification_operation_open_clicked);
+        gtk_widget_class_bind_template_callback (wclass, on_notification_operation_close_clicked);
+
 	g_object_class_install_properties (oclass, NUM_PROPERTIES, properties);
 }
 
